@@ -90,13 +90,10 @@ class APIClient(json.JSONEncoder):
         self.authorize_url = api_server + "/oauth/authorize"
         self.access_token_url = api_server + "/oauth/token"
         self.revoke_url = api_server + "/oauth/revoke"
-        self.revoke_all_url = (
-            api_server + "/a/{client_id}/accounts/{account_id}/revoke-all"
-        )
+        self.application_url = api_server + "/a/{client_id}"
+        self.revoke_all_url = self.application_url + "/accounts/{account_id}/revoke-all"
         self.ip_addresses_url = api_server + "/a/{client_id}/ip_addresses"
-        self.token_info_url = (
-            api_server + "/a/{client_id}/accounts/{account_id}/token-info"
-        )
+        self.token_info_url = self.application_url + "/accounts/{account_id}/token-info"
 
         self.client_secret = client_secret
         self.client_id = client_id
@@ -172,7 +169,16 @@ class APIClient(json.JSONEncoder):
         url = URLObject(self.authorize_url).add_query_params(args.items())
         return str(url)
 
-    def token_for_code(self, code):
+    def send_authorization(self, code):
+        """
+        Exchanges an authorization code for an access token.
+
+        Args:
+            code (str): The authorization code returned from authenticating the user
+
+        Returns:
+            dict: The response from the API containing the access token
+        """
         args = {
             "client_id": self.client_id,
             "client_secret": self.client_secret,
@@ -189,7 +195,11 @@ class APIClient(json.JSONEncoder):
             self.access_token_url, data=urlencode(args), headers=headers
         ).json()
 
-        self.access_token = resp[u"access_token"]
+        self.access_token = resp["access_token"]
+        return resp
+
+    def token_for_code(self, code):
+        self.send_authorization(code)
         return self.access_token
 
     def is_opensource_api(self):
@@ -198,6 +208,31 @@ class APIClient(json.JSONEncoder):
 
         return False
 
+    def application_details(self):
+        application_details_url = self.application_url.format(client_id=self.client_id)
+        resp = self.admin_session.get(application_details_url)
+        _validate(resp).json()
+        return resp.json()
+
+    def update_application_details(
+        self, application_name=None, icon_url=None, redirect_uris=None
+    ):
+        application_details_url = self.application_url.format(client_id=self.client_id)
+        data = {}
+        if application_name is not None:
+            data["application_name"] = application_name
+        if icon_url is not None:
+            data["icon_url"] = icon_url
+        if redirect_uris is not None:
+            data["redirect_uris"] = redirect_uris
+
+        headers = {"Content-Type": "application/json"}
+        headers.update(self.admin_session.headers)
+        resp = self.admin_session.put(
+            application_details_url, json=data, headers=headers
+        )
+        return _validate(resp).json()
+
     def revoke_token(self):
         resp = self.session.post(self.revoke_url)
         _validate(resp)
@@ -619,25 +654,25 @@ class APIClient(json.JSONEncoder):
         return result.json()
 
     def _update_resource(self, cls, id, data, **kwargs):
-        result = self._put_resource(cls, id, data, kwargs)
+        result = self._put_resource(cls, id, data, **kwargs)
         return cls.create(self, **result)
 
     def _post_resource(self, cls, id, method_name, data, path=None):
         if path is None:
             path = cls.collection_name
         path = "/{}".format(path) if path else ""
+        id = "/{}".format(id) if id else ""
+        method = "/{}".format(method_name) if method_name else ""
         if not cls.api_root:
-            url_path = "{name}/{id}/{method}".format(
-                name=path, id=id, method=method_name
-            )
+            url_path = "{name}{id}{method}".format(name=path, id=id, method=method)
         else:
             # Management method.
-            url_path = "/{prefix}/{client_id}{path}/{id}/{method}".format(
+            url_path = "/{prefix}/{client_id}{path}{id}{method}".format(
                 prefix=cls.api_root,
                 client_id=self.client_id,
                 path=path,
                 id=id,
-                method=method_name,
+                method=method,
             )
 
         url = URLObject(self.api_server).with_path(url_path)
